# coding: utf-8

# flake8: noqa
"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from crypticorn.trade.client.models.actions_count import ActionsCount
from crypticorn.trade.client.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.trade.client.models.api_error_level import ApiErrorLevel
from crypticorn.trade.client.models.api_error_type import ApiErrorType
from crypticorn.trade.client.models.bot import Bot
from crypticorn.trade.client.models.bot_create import BotCreate
from crypticorn.trade.client.models.bot_status import BotStatus
from crypticorn.trade.client.models.bot_update import BotUpdate
from crypticorn.trade.client.models.exception_detail import ExceptionDetail
from crypticorn.trade.client.models.exchange import Exchange
from crypticorn.trade.client.models.exchange_info import ExchangeInfo
from crypticorn.trade.client.models.exchange_key import ExchangeKey
from crypticorn.trade.client.models.exchange_key_balance import ExchangeKeyBalance
from crypticorn.trade.client.models.exchange_key_create import ExchangeKeyCreate
from crypticorn.trade.client.models.exchange_key_update import ExchangeKeyUpdate
from crypticorn.trade.client.models.execution_ids import ExecutionIds
from crypticorn.trade.client.models.futures_balance import FuturesBalance
from crypticorn.trade.client.models.futures_trading_action import FuturesTradingAction
from crypticorn.trade.client.models.futures_trading_action_create import (
    FuturesTradingActionCreate,
)
from crypticorn.trade.client.models.margin_mode import MarginMode
from crypticorn.trade.client.models.market_type import MarketType
from crypticorn.trade.client.models.notification import Notification
from crypticorn.trade.client.models.notification_create import NotificationCreate
from crypticorn.trade.client.models.notification_update import NotificationUpdate
from crypticorn.trade.client.models.order import Order
from crypticorn.trade.client.models.order_status import OrderStatus
from crypticorn.trade.client.models.orders_count import OrdersCount
from crypticorn.trade.client.models.paginated_response_futures_trading_action import (
    PaginatedResponseFuturesTradingAction,
)
from crypticorn.trade.client.models.paginated_response_order import (
    PaginatedResponseOrder,
)
from crypticorn.trade.client.models.pn_l import PnL
from crypticorn.trade.client.models.post_futures_action import PostFuturesAction
from crypticorn.trade.client.models.spot_balance import SpotBalance
from crypticorn.trade.client.models.spot_trading_action_create import (
    SpotTradingActionCreate,
)
from crypticorn.trade.client.models.strategy import Strategy
from crypticorn.trade.client.models.strategy_create import StrategyCreate
from crypticorn.trade.client.models.strategy_exchange_info import StrategyExchangeInfo
from crypticorn.trade.client.models.strategy_update import StrategyUpdate
from crypticorn.trade.client.models.tpsl import TPSL
from crypticorn.trade.client.models.tpsl_create import TPSLCreate
from crypticorn.trade.client.models.trading_action_type import TradingActionType



"""Utilities for handling paginated API responses, cursor-based pagination, filtering, sorting, etc."""

import math
from typing import Annotated, Any, Generic, Literal, Optional, Type, TypeVar

from pydantic import BaseModel, Field, model_validator

T = TypeVar("T")


class PaginatedResponse(BaseModel, Generic[T]):
    """Pydantic model for paginated response
    >>> @router.get("", operation_id="getOrders")
    >>> async def get_orders(
    >>>     params: Annotated[PaginationParams, Query()],
    >>> ) -> PaginatedResponse[Order]:
    >>>     ...
    >>> return PaginatedResponse[Order](
            data=orders,
            total=count,
            page=params.page,
            page_size=params.page_size,
            prev=PaginatedResponse.get_prev_page(params.page),
            next=PaginatedResponse.get_next_page(count, params.page_size, params.page),
            last=PaginatedResponse.get_last_page(count, params.page_size),
        )
    """

    data: list[T]
    total: int = Field(description="The total number of items")
    page: int = Field(description="The current page number")
    page_size: int = Field(description="The number of items per page")
    prev: Optional[int] = Field(None, description="The previous page number")
    next: Optional[int] = Field(None, description="The next page number")
    last: Optional[int] = Field(None, description="The last page number")

    @staticmethod
    def get_next_page(total: int, page_size: int, page: int) -> Optional[int]:
        """Get the next page number"""
        if page < math.ceil(total / page_size):
            return page + 1
        return None

    @staticmethod
    def get_prev_page(page: int) -> Optional[int]:
        """Get the previous page number"""
        if page > 1:
            return page - 1
        return None

    @staticmethod
    def get_last_page(total: int, page_size: int) -> int:
        """Get the last page number"""
        return max(1, math.ceil(total / page_size))


class PaginationParams(BaseModel, Generic[T]):
    """Standard pagination parameters for usage in API endpoints. Check the [fastapi docs](https://fastapi.tiangolo.com/tutorial/query-param-models/?h=qu#query-parameters-with-a-pydantic-model) for usage examples.
    You should inherit from this class when adding additional parameters. You should use this class in combination with `PaginatedResponse` to return the paginated response.
    Usage:
    >>> @router.get("", operation_id="getOrders")
    >>> async def get_orders(
    >>>     params: Annotated[PaginationParams[Order], Query()],
    >>> ) -> PaginatedResponse[Order]:
    >>>     ...

    The default size is 10 items per page and there is a `HeavyPaginationParams` class with 100 items per page. You can override this default:
    >>> class LightPaginationParams(PaginationParams):
    >>>     page_size: int = Field(default=5, description="The number of items per page")
    """

    page: Optional[int] = Field(default=1, description="The current page number")
    page_size: Annotated[int, Field(ge=1, le=100)] = Field(
        10, description="The number of items per page. Default is 10, max is 100."
    )


class HeavyPaginationParams(PaginationParams[T]):
    """Pagination parameters with a higher default size. Refer to `PaginationParams` for usage examples."""

    page_size: Annotated[int, Field(ge=1, le=1000)] = Field(
        100, description="The number of items per page. Default is 100, max is 1000."
    )


class SortParams(BaseModel, Generic[T]):
    """Standard sort parameters for usage in API endpoints. Check the [fastapi docs](https://fastapi.tiangolo.com/tutorial/query-param-models/?h=qu#query-parameters-with-a-pydantic-model) for usage examples.
    You should inherit from this class when adding additional parameters.
    Usage:
    >>> @router.get("", operation_id="getOrders")
    >>> async def get_orders(
    >>>     params: Annotated[SortParams[Order], Query()],
    >>> ) -> PaginatedResponse[Order]:
    >>>     ...
    """

    sort_order: Optional[Literal["asc", "desc"]] = Field(
        None, description="The order to sort by"
    )
    sort_by: Optional[str] = Field(None, description="The field to sort by")

    @model_validator(mode="after")
    def validate_sort(self):
        # Extract the generic argument type
        args: tuple = self.__pydantic_generic_metadata__.get("args")
        if not args or not issubclass(args[0], BaseModel):
            raise TypeError(
                "SortParams must be used with a Pydantic BaseModel as a generic parameter"
            )
        if self.sort_by:
            # check if the sort field is valid
            model: Type[BaseModel] = args[0]
            if self.sort_by not in model.model_fields:
                raise ValueError(
                    f"Invalid field: '{self.sort_by}'. Must be one of: {list(model.model_fields)}"
                )
        if self.sort_order and self.sort_order not in ["asc", "desc"]:
            raise ValueError(
                f"Invalid order: '{self.sort_order}' — must be one of: ['asc', 'desc']"
            )
        if (
            self.sort_order
            and self.sort_by is None
            or self.sort_by
            and self.sort_order is None
        ):
            raise ValueError("sort_order and sort_by must be provided together")
        return self


class FilterParams(BaseModel, Generic[T]):
    """Standard filter parameters for usage in API endpoints. Check the [fastapi docs](https://fastapi.tiangolo.com/tutorial/query-param-models/?h=qu#query-parameters-with-a-pydantic-model) for usage examples.
    You should inherit from this class when adding additional parameters.
    Usage:
    >>> @router.get("", operation_id="getOrders")
    >>> async def get_orders(
    >>>     params: Annotated[FilterParams[Order], Query()],
    >>> ) -> PaginatedResponse[Order]:
    >>>     ...
    """

    filter_by: Optional[str] = Field(None, description="The field to filter by")
    filter_value: Optional[str] = Field(None, description="The value to filter with")
    # currently openapi-gen does not support typing.Any in combo with None, so we use str
    # this is fine since the input is a string anyways from the request and the correct type is enforced by the model validator from the filter_by field

    @model_validator(mode="after")
    def validate_filter(self):
        if self.filter_by and not self.filter_value:
            raise ValueError("filter_by and filter_value must be provided together")
        if self.filter_by:
            # Extract the generic argument type
            args: tuple = self.__pydantic_generic_metadata__.get("args")
            if not args or not issubclass(args[0], BaseModel):
                raise TypeError(
                    "FilterParams must be used with a Pydantic BaseModel as a generic parameter"
                )
            # check if the filter field is valid
            model: Type[BaseModel] = args[0]
            if self.filter_by not in model.model_fields:
                raise ValueError(
                    f"Invalid field: '{self.filter_by}'. Must be one of: {list(model.model_fields)}"
                )
            self.filter_value = _enforce_field_type(
                model, self.filter_by, self.filter_value
            )
        return self


class SortFilterParams(SortParams[T], FilterParams[T]):
    """Combines sort and filter parameters. Just a convenience class for when you need to combine sort and filter parameters.
    You should inherit from this class when adding additional parameters.
    Usage:
    >>> @router.get("", operation_id="getOrders")
    >>> async def get_orders(
    >>>     params: Annotated[SortFilterParams[Order], Query()],
    >>> ) -> PaginatedResponse[Order]:
    >>>     ...
    """

    @model_validator(mode="after")
    def validate_sort_filter(self):
        self.validate_sort()
        self.validate_filter()
        return self


class PageFilterParams(PaginationParams[T], FilterParams[T]):
    """Combines pagination and filter parameters. Just a convenience class for when you need to combine pagination and filter parameters.
    You should inherit from this class when adding additional parameters.
    Usage:
    >>> @router.get("", operation_id="getOrders")
    >>> async def get_orders(
    >>>     params: Annotated[PageFilterParams[Order], Query()],
    >>> ) -> PaginatedResponse[Order]:
    >>>     ...
    """

    @model_validator(mode="after")
    def validate_page_filter(self):
        self.validate_filter()
        return self


class PageSortParams(PaginationParams[T], SortParams[T]):
    """Combines pagination and sort parameters. Just a convenience class for when you need to combine pagination and sort parameters.
    You should inherit from this class when adding additional parameters.
    Usage:
    >>> @router.get("", operation_id="getOrders")
    >>> async def get_orders(
    >>>     params: Annotated[PageSortParams[Order], Query()],
    >>> ) -> PaginatedResponse[Order]:
    >>>     ...
    """

    @model_validator(mode="after")
    def validate_page_sort(self):
        self.validate_sort()
        return self


class PageSortFilterParams(
    PaginationParams[T],
    SortParams[T],
    FilterParams[T],
):
    """Combines pagination, filter, and sort parameters. Just a convenience class for when you need to combine pagination, filter, and sort parameters.
    You should inherit from this class when adding additional parameters.
    Usage:
    >>> @router.get("", operation_id="getOrders")
    >>> async def get_orders(
    >>>     params: Annotated[PageSortFilterParams[Order], Query()],
    >>> ) -> PaginatedResponse[Order]:
    >>>     ...
    """

    @model_validator(mode="after")
    def validate_filter_combo(self):
        self.validate_filter()
        self.validate_sort()
        return self


class HeavyPageSortFilterParams(
    HeavyPaginationParams[T], FilterParams[T], SortParams[T]
):
    """Combines heavy pagination, filter, and sort parameters. Just a convenience class for when you need to combine heavy pagination, filter, and sort parameters.
    You should inherit from this class when adding additional parameters.
    Usage:
    >>> @router.get("", operation_id="getOrders")
    >>> async def get_orders(
    >>>     params: Annotated[HeavyPageSortFilterParams[Order], Query()],
    >>> ) -> PaginatedResponse[Order]:
    >>>     ...
    """

    @model_validator(mode="after")
    def validate_heavy_page_sort_filter(self):
        self.validate_filter()
        self.validate_sort()
        return self


def _enforce_field_type(model: Type[BaseModel], field_name: str, value: Any) -> Any:
    """
    Coerce or validate `value` to match the type of `field_name` on the given `model`. Should be used after checking that the field is valid.

    :param model: The Pydantic model.
    :param field_name: The name of the field to match.
    :param value: The value to validate or coerce.

    :return: The value cast to the expected type.

    :raises: ValueError: If the field doesn't exist or coercion fails.
    """
    expected_type = model.model_fields[field_name].annotation

    if isinstance(value, expected_type):
        return value

    try:
        return expected_type(value)
    except Exception:
        raise ValueError(
            f"Expected {expected_type} for field {field_name}, got {type(value)}"
        )



"""General utility functions and client helper methods used across the codebase."""

from typing import Any


def optional_import(module_name: str, extra_name: str) -> Any:
    """
    Tries to import a module. Raises `ImportError` if not found with a message to install the extra dependency.
    """
    try:
        return __import__(module_name)
    except ImportError as e:
        raise ImportError(
            f"Optional dependency '{module_name}' is required for this feature. "
            f"Install it with: pip install crypticorn[{extra_name}]"
        ) from e



# coding: utf-8

"""
Hive AI API

API for Hive AI model training and evaluation

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class TargetType(str, Enum):
    """
    Type classification for AI prediction targets (continuous vs binary outcomes).
    """

    """
    allowed enum values
    """
    CONTINUOUS = "continuous"
    BINARY = "binary"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of TargetType from a JSON string"""
        return cls(json.loads(json_str))



# coding: utf-8

"""
Klines Service API

API for retrieving OHLCV data, funding rates, and symbol information from Binance.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing_extensions import Self


class SymbolInfo(BaseModel):
    """
    Comprehensive symbol information for UDF charting library compatibility.
    """  # noqa: E501

    name: StrictStr = Field(description="Symbol name identifier")
    exchange_traded: StrictStr = Field(
        description="Exchange where symbol is traded", alias="exchange-traded"
    )
    exchange_listed: StrictStr = Field(
        description="Exchange where symbol is listed", alias="exchange-listed"
    )
    timezone: StrictStr = Field(description="Timezone for symbol trading hours")
    minmov: StrictInt = Field(description="Minimum price movement")
    minmov2: StrictInt = Field(description="Secondary minimum price movement")
    pointvalue: StrictInt = Field(description="Point value for price calculations")
    session: StrictStr = Field(description="Trading session hours")
    has_intraday: StrictBool = Field(description="Whether intraday data is available")
    has_no_volume: StrictBool = Field(description="Whether volume data is unavailable")
    description: StrictStr = Field(description="Human-readable symbol description")
    type: StrictStr = Field(description="Symbol type classification")
    supported_resolutions: List[StrictStr] = Field(
        description="List of supported time resolutions"
    )
    pricescale: StrictInt = Field(description="Price scale factor for decimal places")
    ticker: StrictStr = Field(description="Symbol ticker identifier")
    __properties: ClassVar[List[str]] = [
        "name",
        "exchange-traded",
        "exchange-listed",
        "timezone",
        "minmov",
        "minmov2",
        "pointvalue",
        "session",
        "has_intraday",
        "has_no_volume",
        "description",
        "type",
        "supported_resolutions",
        "pricescale",
        "ticker",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SymbolInfo from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SymbolInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "name": obj.get("name"),
                "exchange-traded": obj.get("exchange-traded"),
                "exchange-listed": obj.get("exchange-listed"),
                "timezone": obj.get("timezone"),
                "minmov": obj.get("minmov"),
                "minmov2": obj.get("minmov2"),
                "pointvalue": obj.get("pointvalue"),
                "session": obj.get("session"),
                "has_intraday": obj.get("has_intraday"),
                "has_no_volume": obj.get("has_no_volume"),
                "description": obj.get("description"),
                "type": obj.get("type"),
                "supported_resolutions": obj.get("supported_resolutions"),
                "pricescale": obj.get("pricescale"),
                "ticker": obj.get("ticker"),
            }
        )
        return _obj



import logging
import warnings
from enum import EnumMeta

from crypticorn.common.warnings import CrypticornDeprecatedSince28

_logger = logging.getLogger("crypticorn")


class ValidateEnumMixin:
    """
    Mixin for validating enum values manually.

    ⚠️ Note:
    This does NOT enforce validation automatically on enum creation.
    It's up to the developer to call `Class.validate(value)` where needed.

    Usage:
        >>> class Color(ValidateEnumMixin, StrEnum):
        >>>     RED = "red"
        >>>     GREEN = "green"

        >>> Color.validate("red")     # True
        >>> Color.validate("yellow")  # False

    Order of inheritance matters — the mixin must come first.
    """

    @classmethod
    def validate(cls, value) -> bool:
        """Validate if a value is in the enum. True if so, False otherwise."""
        try:
            cls(value)
            return True
        except ValueError:
            return False


# This Mixin will be removed in a future version. And has no effect from now on
class ExcludeEnumMixin:
    """(deprecated) Mixin to exclude enum from OpenAPI schema. We use this to avoid duplicating enums when generating client code from the openapi spec."""

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        if cls.__name__.startswith("ExcludeEnumMixin"):
            warnings.warn(
                "The `ExcludeEnumMixin` class is deprecated. Should be removed from enums inheriting this class.",
                category=CrypticornDeprecatedSince28,
            )

    @classmethod
    def __get_pydantic_json_schema__(cls, core_schema, handler):
        schema = handler(core_schema)
        # schema.pop("enum", None)
        return schema


class ApiErrorFallback(EnumMeta):
    """Fallback for enum members that are not yet published to PyPI."""

    def __getattr__(cls, name):
        # Let Pydantic/internal stuff pass silently ! fragile
        if name.startswith("__") or name.startswith("_pytest"):
            raise AttributeError(name)
        _logger.warning(
            f"Unknown enum member '{name}' - update crypticorn package or check for typos"
        )
        return cls.UNKNOWN_ERROR



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictStr
from typing_extensions import Self

from crypticorn.trade.client.models.futures_balance import FuturesBalance
from crypticorn.trade.client.models.spot_balance import SpotBalance


class ExchangeKeyBalance(BaseModel):
    """
    Balance information for an exchange API key including futures and spot balances.
    """  # noqa: E501

    api_key_id: StrictStr = Field(description="API key ID.")
    futures: FuturesBalance = Field(description="Futures balance information.")
    spot: List[SpotBalance] = Field(description="Spot balance information.")
    __properties: ClassVar[List[str]] = ["api_key_id", "futures", "spot"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ExchangeKeyBalance from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of futures
        if self.futures:
            _dict["futures"] = self.futures.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in spot (list)
        _items = []
        if self.spot:
            for _item_spot in self.spot:
                if _item_spot:
                    _items.append(_item_spot.to_dict())
            _dict["spot"] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ExchangeKeyBalance from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "api_key_id": obj.get("api_key_id"),
                "futures": (
                    FuturesBalance.from_dict(obj["futures"])
                    if obj.get("futures") is not None
                    else None
                ),
                "spot": (
                    [SpotBalance.from_dict(_item) for _item in obj["spot"]]
                    if obj.get("spot") is not None
                    else None
                ),
            }
        )
        return _obj



# coding: utf-8

"""
Hive AI API

API for Hive AI model training and evaluation

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
from enum import Enum

from typing_extensions import Self


class Coins(str, Enum):
    """
    Cryptocurrency identifiers for AI training data. Uses obfuscated identifiers for public API access.
    """

    """
    allowed enum values
    """
    ENUM_1 = "1"
    ENUM_2 = "2"
    ENUM_3 = "3"
    ENUM_4 = "4"
    ENUM_5 = "5"
    ENUM_6 = "6"
    ENUM_7 = "7"
    ENUM_8 = "8"
    ENUM_9 = "9"
    ENUM_10 = "10"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Coins from a JSON string"""
        return cls(json.loads(json_str))



import importlib.resources
import subprocess
from pathlib import Path

import click

import crypticorn.cli.templates as templates


def get_git_root() -> Path:
    """Get the root directory of the git repository."""
    try:
        return Path(
            subprocess.check_output(
                ["git", "rev-parse", "--show-toplevel"], text=True
            ).strip()
        )
    except Exception:
        return Path.cwd()


def copy_template(template_name: str, target_path: Path):
    """Copy a template file to the target path."""
    with (
        importlib.resources.files(templates)
        .joinpath(template_name)
        .open("r") as template_file
    ):
        content = template_file.read()

    target_path.parent.mkdir(parents=True, exist_ok=True)
    with target_path.open("w") as f:
        f.write(content)

    click.secho(f"✅ Created: {target_path}", fg="green")


def check_file_exists(path: Path, force: bool):
    if path.exists() and not force:
        click.secho("File already exists, use --force / -f to overwrite", fg="red")
        return False
    return True


@click.group()
def init_group():
    """Initialize files like CI configs, linters, etc."""
    pass


@init_group.command("ruff")
@click.option("-f", "--force", is_flag=True, help="Force overwrite the ruff.yml")
def init_ruff(force):
    """Add .github/workflows/ruff.yml"""
    root = get_git_root()
    target = root / ".github/workflows/ruff.yml"
    if target.exists() and not force:
        click.secho("File already exists, use --force / -f to overwrite", fg="red")
        return
    copy_template("ruff.yml", target)


@init_group.command("docker")
@click.option(
    "-o", "--output", type=click.Path(), help="Custom output path for the Dockerfile"
)
@click.option("-f", "--force", is_flag=True, help="Force overwrite the Dockerfile")
def init_docker(output, force):
    """Add Dockerfile"""
    root = get_git_root()
    if output and Path(output).is_file():
        click.secho("Output path is a file, please provide a directory path", fg="red")
        return
    target = (Path(output) if output else root) / "Dockerfile"
    if not check_file_exists(target, force):
        return
    copy_template("Dockerfile", target)
    click.secho("Make sure to update the Dockerfile", fg="yellow")


@init_group.command("auth")
@click.option(
    "-o", "--output", type=click.Path(), help="Custom output path for the auth handler"
)
@click.option("-f", "--force", is_flag=True, help="Force overwrite the auth handler")
def init_auth(output, force):
    """Add auth.py with auth handler. Everything you need to start using the auth service."""
    root = get_git_root()
    if output and Path(output).is_file():
        click.secho("Output path is a file, please provide a directory path", fg="red")
        return
    target = (Path(output) if output else root) / "auth.py"
    if not check_file_exists(target, force):
        return
    copy_template("auth.py", target)
    click.secho(
        """
    Make sure to update the .env and .env.example files with:
        IS_DOCKER=0
        API_ENV=local
    and the docker-compose.yml file with:
        environment:
            - IS_DOCKER=1
    and the .github/workflows/main.yaml file with:
        env:
            API_ENV: ${{ github.ref == 'refs/heads/main' && 'prod' || 'dev' }}
    """,
        fg="yellow",
    )


@init_group.command("dependabot")
@click.option("-f", "--force", is_flag=True, help="Force overwrite the dependabot.yml")
def init_dependabot(force):
    """Add dependabot.yml"""
    root = get_git_root()
    target = root / ".github/dependabot.yml"
    if not check_file_exists(target, force):
        return
    copy_template("dependabot.yml", target)


@init_group.command("merge-env")
@click.option("-f", "--force", is_flag=True, help="Force overwrite the .env file")
def init_merge_env(force):
    """Add script to merge environment and file variables into .env"""
    root = get_git_root()
    target = root / "scripts/merge-env.sh"
    if not check_file_exists(target, force):
        return
    copy_template("merge-env.sh", target)



from __future__ import annotations

from typing import TYPE_CHECKING, Optional

from crypticorn.pay import (
    AccessApi,
    ApiClient,
    Configuration,
    CouponsApi,
    InvoicesApi,
    NOWPaymentsApi,
    PaymentsApi,
    ProductsApi,
    StatusApi,
    StripeApi,
    TokenApi,
)

if TYPE_CHECKING:
    from aiohttp import ClientSession


class PayClient:
    """
    A client for interacting with the Crypticorn Pay API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        self.now = NOWPaymentsApi(self.base_client, is_sync=is_sync)
        self.status = StatusApi(self.base_client, is_sync=is_sync)
        self.payments = PaymentsApi(self.base_client, is_sync=is_sync)
        self.products = ProductsApi(self.base_client, is_sync=is_sync)
        self.coupons = CouponsApi(self.base_client, is_sync=is_sync)
        self.token = TokenApi(self.base_client, is_sync=is_sync)
        self.invoices = InvoicesApi(self.base_client, is_sync=is_sync)
        self.stripe = StripeApi(self.base_client, is_sync=is_sync)
        self.access = AccessApi(self.base_client, is_sync=is_sync)



from enum import IntEnum

import pytest

from crypticorn.common.enums import MarketType, ValidateEnumMixin


class MockInt(ValidateEnumMixin, IntEnum):
    SPOT = 1
    FUTURES = 2


@pytest.fixture
def int_enum() -> MockInt:
    return MockInt


def test_str_enum_validation():
    """Test that the enum validation works"""
    # Test valid string values
    assert MarketType.validate("spot") is True
    assert MarketType.validate("futures") is True

    # Test invalid string values
    assert MarketType.validate("SPOT") is False
    assert MarketType.validate("invalid") is False

    # Test enum values
    assert MarketType.validate(MarketType.SPOT) is True
    assert MarketType.validate(MarketType.FUTURES) is True

    # Test enum construction
    assert MarketType("spot") == MarketType.SPOT
    assert MarketType("futures") == MarketType.FUTURES

    # Test invalid enum construction
    with pytest.raises(ValueError):
        MarketType("SPOT")
    with pytest.raises(ValueError):
        MarketType("invalid")


def test_int_enum_validation(int_enum):
    """Test that the enum validation works"""
    assert int_enum.validate(int_enum.SPOT) is True
    assert int_enum.validate(int_enum.FUTURES) is True



import logging
import warnings

from crypticorn.client import ApiClient, AsyncClient, SyncClient
from crypticorn.common.logging import configure_logging

warnings.filterwarnings("default", "", DeprecationWarning)
configure_logging()
logging.captureWarnings(True)
# TODO: remove logging in next major release

__all__ = ["AsyncClient", "SyncClient", "ApiClient"]



import asyncio
import gc
import warnings

import pytest
from aiohttp import ClientSession

from crypticorn.client import AsyncClient


@pytest.mark.asyncio
async def test_custom_http_client_injection():
    custom_session = ClientSession()

    client = AsyncClient(http_client=custom_session)

    # Subclients should have received the custom session immediately (sync context)
    subclient = client._services["trade"]
    assert subclient.base_client.rest_client.pool_manager is custom_session

    await client.close()
    assert not custom_session.closed  # Client did not own it

    await custom_session.close()


@pytest.mark.asyncio
async def test_lazy_http_client_creation():
    client = AsyncClient()
    assert client._http_client is None

    client._ensure_session()

    assert isinstance(client._http_client, ClientSession)
    subclient = client._services["trade"]
    assert subclient.base_client.rest_client.pool_manager is client._http_client

    await client.close()
    assert client._http_client is None  # It should have been closed


@pytest.mark.asyncio
async def test_close_custom_http_client_not_owned():
    custom_session = ClientSession()
    client = AsyncClient(http_client=custom_session)

    await client.close()
    assert not custom_session.closed  # Still open since it wasn't owned by us

    await custom_session.close()


@pytest.mark.asyncio
async def test_close_owned_http_client():
    client = AsyncClient()
    client._ensure_session()
    session = client._http_client

    await client.close()
    assert session.closed  # session should be closed since we own it


@pytest.mark.asyncio
async def test_unclosed_owned_session_warns():
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")  # Catch all warnings

        client = AsyncClient()
        await client.trade.status.ping()

        # Intentionally forget to close
        del client
        await asyncio.sleep(0.1)  # Let __del__ run

        # Look for aiohttp's unclosed session warning
        unclosed_warnings = [
            warn for warn in w if "Unclosed client session" in str(warn.message)
        ]
        assert unclosed_warnings, "Expected unclosed client session warning"


@pytest.mark.asyncio
async def test_custom_session_not_closed_by_client():
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        custom_session = ClientSession()
        client = AsyncClient(http_client=custom_session)
        await client.trade.status.ping()

        # Don't close the custom session
        del client
        del custom_session

        # Force cleanup (otherwise it's unpredicatable when the warning will be raised)
        await asyncio.sleep(0.1)
        gc.collect()
        await asyncio.sleep(0.1)

        unclosed_warnings = [
            warn
            for warn in w
            if "Unclosed client session" in str(warn.message)
            or "Unclosed connector" in str(warn.message)
        ]
        assert unclosed_warnings, "Expected unclosed client session warning"


@pytest.mark.asyncio
async def test_context_manager_usage():
    async with AsyncClient() as client:
        assert isinstance(client._http_client, ClientSession)
        subclient = client._services["trade"]
        assert subclient.base_client.rest_client.pool_manager is client._http_client

    # Confirm session is closed after context manager
    assert client._http_client is None



# coding: utf-8

# flake8: noqa

"""
Klines Service API

API for retrieving OHLCV data, funding rates, and symbol information from Binance.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


__version__ = "1.0.0"

# import apis into sdk package
from crypticorn.klines.client.api.admin_api import AdminApi
from crypticorn.klines.client.api.change_in_timeframe_api import ChangeInTimeframeApi
from crypticorn.klines.client.api.funding_rates_api import FundingRatesApi
from crypticorn.klines.client.api.ohlcv_data_api import OHLCVDataApi
from crypticorn.klines.client.api.status_api import StatusApi
from crypticorn.klines.client.api.symbols_api import SymbolsApi
from crypticorn.klines.client.api.udf_api import UDFApi
from crypticorn.klines.client.api_client import ApiClient

# import ApiClient
from crypticorn.klines.client.api_response import ApiResponse
from crypticorn.klines.client.configuration import Configuration
from crypticorn.klines.client.exceptions import (
    ApiAttributeError,
    ApiException,
    ApiKeyError,
    ApiTypeError,
    ApiValueError,
    OpenApiException,
)

# import models into sdk package
from crypticorn.klines.client.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.klines.client.models.api_error_level import ApiErrorLevel
from crypticorn.klines.client.models.api_error_type import ApiErrorType
from crypticorn.klines.client.models.change_in_timeframe import ChangeInTimeframe
from crypticorn.klines.client.models.exception_detail import ExceptionDetail
from crypticorn.klines.client.models.funding_rate import FundingRate
from crypticorn.klines.client.models.funding_rate_response import FundingRateResponse
from crypticorn.klines.client.models.internal_exchange import InternalExchange
from crypticorn.klines.client.models.log_level import LogLevel
from crypticorn.klines.client.models.market_type import MarketType
from crypticorn.klines.client.models.ohlcv import OHLCV
from crypticorn.klines.client.models.resolution import Resolution
from crypticorn.klines.client.models.search_symbol import SearchSymbol
from crypticorn.klines.client.models.sort_direction import SortDirection
from crypticorn.klines.client.models.symbol_group import SymbolGroup
from crypticorn.klines.client.models.symbol_info import SymbolInfo
from crypticorn.klines.client.models.symbol_type import SymbolType
from crypticorn.klines.client.models.timeframe import Timeframe
from crypticorn.klines.client.models.udf_config import UDFConfig



try:
    from enum import StrEnum
except ImportError:
    from strenum import StrEnum


class AnsiColors(StrEnum):
    """Provides a collection of ANSI color codes for terminal text formatting, including regular, bright, and bold text colors. Useful for creating colorful and readable console output."""

    # Regular Text Colors
    BLACK = "\033[30m"  # black
    RED = "\033[31m"  # red
    GREEN = "\033[32m"  # green
    YELLOW = "\033[33m"  # yellow
    BLUE = "\033[34m"  # blue
    MAGENTA = "\033[35m"  # magenta
    CYAN = "\033[36m"  # cyan
    WHITE = "\033[37m"  # white

    # Bright Text Colors
    BLACK_BRIGHT = "\033[90m"  # black_bright
    RED_BRIGHT = "\033[91m"  # red_bright
    GREEN_BRIGHT = "\033[92m"  # green_bright
    YELLOW_BRIGHT = "\033[93m"  # yellow_bright
    BLUE_BRIGHT = "\033[94m"  # blue_bright
    MAGENTA_BRIGHT = "\033[95m"  # magenta_bright
    CYAN_BRIGHT = "\033[96m"  # cyan_bright
    WHITE_BRIGHT = "\033[97m"  # white_bright

    # Bold Text Colors
    BLACK_BOLD = "\033[1;30m"  # black_bold
    RED_BOLD = "\033[1;31m"  # red_bold
    GREEN_BOLD = "\033[1;32m"  # green_bold
    YELLOW_BOLD = "\033[1;33m"  # yellow_bold
    BLUE_BOLD = "\033[1;34m"  # blue_bold
    MAGENTA_BOLD = "\033[1;35m"  # magenta_bold
    CYAN_BOLD = "\033[1;36m"  # cyan_bold
    WHITE_BOLD = "\033[1;37m"  # white_bold

    # Reset Color
    RESET = "\033[0m"



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing_extensions import Annotated, Self

from crypticorn.trade.client.models.api_error_identifier import ApiErrorIdentifier
from crypticorn.trade.client.models.bot_status import BotStatus


class Bot(BaseModel):
    """
    Trading bot model for read operations.
    """  # noqa: E501

    user_id: StrictStr = Field(description="UID for the user")
    created_at: StrictInt = Field(description="Timestamp of creation")
    updated_at: StrictInt = Field(description="Timestamp of last update")
    id: StrictStr = Field(description="Unique identifier for the resource")
    name: StrictStr = Field(description="Name of the bot")
    allocation: Annotated[int, Field(strict=True, ge=1)] = Field(
        description="Initial allocation for the bot"
    )
    status: BotStatus = Field(description="Status of the bot")
    strategy_id: StrictStr = Field(
        description="UID for the trading strategy used by the bot"
    )
    api_key_id: StrictStr = Field(description="UID for the API key")
    status_code: Optional[ApiErrorIdentifier] = None
    current_allocation: Optional[StrictStr] = Field(
        default="0",
        description="Initial allocation for the bot + accumulated PnL of the orders after the last allocation change",
    )
    current_exposure: Optional[StrictStr] = Field(
        default="0",
        description="Current exposure of the bot, aka. the sum of the absolute values of the open positions",
    )
    __properties: ClassVar[List[str]] = [
        "user_id",
        "created_at",
        "updated_at",
        "id",
        "name",
        "allocation",
        "status",
        "strategy_id",
        "api_key_id",
        "status_code",
        "current_allocation",
        "current_exposure",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Bot from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if status_code (nullable) is None
        # and model_fields_set contains the field
        if self.status_code is None and "status_code" in self.model_fields_set:
            _dict["status_code"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Bot from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "user_id": obj.get("user_id"),
                "created_at": obj.get("created_at"),
                "updated_at": obj.get("updated_at"),
                "id": obj.get("id"),
                "name": obj.get("name"),
                "allocation": obj.get("allocation"),
                "status": obj.get("status"),
                "strategy_id": obj.get("strategy_id"),
                "api_key_id": obj.get("api_key_id"),
                "status_code": obj.get("status_code"),
                "current_allocation": (
                    obj.get("current_allocation")
                    if obj.get("current_allocation") is not None
                    else "0"
                ),
                "current_exposure": (
                    obj.get("current_exposure")
                    if obj.get("current_exposure") is not None
                    else "0"
                ),
            }
        )
        return _obj



import pytest

from crypticorn.client import SyncClient


def test_client_basic_instantiation():
    """Test basic client instantiation without errors."""
    client = SyncClient()
    assert client is not None
    assert client._http_client is None
    client.close()


def test_client_manual_close():
    """Test manual close of client."""
    client = SyncClient()

    assert client._http_client is None

    # Close manually
    client.close()
    assert client._http_client is None


def test_client_ping_functionality():
    """Test that ping works correctly."""
    client = SyncClient()

    try:
        response = client.trade.status.ping()
        # Should not raise any exceptions
        assert response is not None
    finally:
        client.close()


def test_client_multiple_service_access():
    """Test accessing multiple services without errors."""
    client = SyncClient()

    try:
        assert client._http_client is None
        # Access multiple services to ensure they're properly initialized
        subclient = client._services["trade"]
        assert subclient is not None
        assert subclient.base_client.rest_client.pool_manager is None
    finally:
        client.close()


def test_client_no_exceptions_on_basic_operations():
    """Test that basic operations don't raise unexpected exceptions."""
    client = SyncClient()

    try:
        # Access a service
        trade_client = client.trade
        assert trade_client is not None

        # Ping should work
        response = client.trade.status.ping()
        assert response is not None

    except Exception as e:
        pytest.fail(f"Unexpected exception raised: {e}")
    finally:
        client.close()


def test_client_service_lazy_initialization():
    """Test that services are lazily initialized."""
    client = SyncClient()

    # But HTTP client should be None until first use
    assert client._http_client is None
    # First access should create the session
    client.trade.status.ping()
    assert client._http_client is None

    client.close()
    assert client._http_client is None


def test_client_concurrent_operations():
    """Test that client handles concurrent operations correctly."""
    client = SyncClient()

    responses = []
    for _ in range(5):
        responses.append(client.trade.status.ping())  # ✅ Each call properly managed

    client.close()  # ✅ Reliable cleanup
    assert client._http_client is None



import logging
import os

import dotenv
from crypticorn_utils import ApiEnv as ApiEnv
from crypticorn_utils import AuthHandler as AuthHandler
from crypticorn_utils import BaseUrl as BaseUrl
from crypticorn_utils import Scope as Scope
from crypticorn_utils import Verify200Response as Verify200Response
from fastapi import Security as Security

dotenv.load_dotenv()

logger = logging.getLogger(__name__)

DOCKER_ENV = os.getenv("IS_DOCKER", "0")
API_ENV = os.getenv("API_ENV")

if not API_ENV:
    raise ValueError(
        "API_ENV is not set. Please set it to 'prod', 'dev' or 'local' in .env (of type ApiEnv)."
    )

if DOCKER_ENV == "0":
    logger.info(f"Using {API_ENV} environment")
    base_url = BaseUrl.from_env(ApiEnv(API_ENV))
else:
    base_url = BaseUrl.DOCKER
    logger.info("Using docker environment")

auth_handler = AuthHandler(base_url=base_url)
logger.info(f"Auth URL: {auth_handler.client.config.host}")



# coding: utf-8

"""
Payment API

API for accepting payments and storing subscriptions

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set, Union

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictBool,
    StrictInt,
    StrictStr,
    field_validator,
)
from typing_extensions import Annotated, Self


class Coupon(BaseModel):
    """
    Model for reading a coupon
    """  # noqa: E501

    created_at: StrictInt = Field(description="Timestamp of creation")
    updated_at: StrictInt = Field(description="Timestamp of last update")
    id: StrictStr = Field(description="Unique identifier for the resource")
    code: Optional[Annotated[str, Field(min_length=4, strict=True, max_length=32)]] = (
        None
    )
    name: StrictStr = Field(
        description="A name for the coupon, e.g. 'Black Friday 2025'"
    )
    discount: Union[
        Annotated[float, Field(le=1.0, strict=True, ge=0.0)],
        Annotated[int, Field(le=1, strict=True, ge=0)],
    ] = Field(description="Discount percentage as a decimal")
    valid_until: Optional[StrictInt] = None
    valid_from: Optional[StrictInt] = None
    usage_limit: Optional[StrictInt] = None
    products: Optional[List[StrictStr]] = None
    is_active: Optional[StrictBool] = Field(
        default=True, description="Coupon is active"
    )
    usage_count: StrictInt = Field(description="Coupon usage count")
    payment_required: StrictBool = Field(
        description="Whether the coupon requires a payment. If the discount is not 100%, the coupon requires a payment."
    )
    is_valid: StrictBool = Field(
        description="Coupon is valid if it is active, not expired, and has not reached the usage limit"
    )
    __properties: ClassVar[List[str]] = [
        "created_at",
        "updated_at",
        "id",
        "code",
        "name",
        "discount",
        "valid_until",
        "valid_from",
        "usage_limit",
        "products",
        "is_active",
        "usage_count",
        "payment_required",
        "is_valid",
    ]

    @field_validator("code")
    def code_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^[A-Z0-9]+$", value):
            raise ValueError(r"must validate the regular expression /^[A-Z0-9]+$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Coupon from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set(
            [
                "payment_required",
                "is_valid",
            ]
        )

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if code (nullable) is None
        # and model_fields_set contains the field
        if self.code is None and "code" in self.model_fields_set:
            _dict["code"] = None

        # set to None if valid_until (nullable) is None
        # and model_fields_set contains the field
        if self.valid_until is None and "valid_until" in self.model_fields_set:
            _dict["valid_until"] = None

        # set to None if valid_from (nullable) is None
        # and model_fields_set contains the field
        if self.valid_from is None and "valid_from" in self.model_fields_set:
            _dict["valid_from"] = None

        # set to None if usage_limit (nullable) is None
        # and model_fields_set contains the field
        if self.usage_limit is None and "usage_limit" in self.model_fields_set:
            _dict["usage_limit"] = None

        # set to None if products (nullable) is None
        # and model_fields_set contains the field
        if self.products is None and "products" in self.model_fields_set:
            _dict["products"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Coupon from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "created_at": obj.get("created_at"),
                "updated_at": obj.get("updated_at"),
                "id": obj.get("id"),
                "code": obj.get("code"),
                "name": obj.get("name"),
                "discount": obj.get("discount"),
                "valid_until": obj.get("valid_until"),
                "valid_from": obj.get("valid_from"),
                "usage_limit": obj.get("usage_limit"),
                "products": obj.get("products"),
                "is_active": (
                    obj.get("is_active") if obj.get("is_active") is not None else True
                ),
                "usage_count": obj.get("usage_count"),
                "payment_required": obj.get("payment_required"),
                "is_valid": obj.get("is_valid"),
            }
        )
        return _obj



# coding: utf-8

"""
DEX AI API

API for DEX AI

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional, Set

from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing_extensions import Self

from crypticorn.dex.client.models.risk import Risk
from crypticorn.dex.client.models.token_detail import TokenDetail


class TokenData(BaseModel):
    """
    Main model for complete token data
    """  # noqa: E501

    token: TokenDetail = Field(description="Token details")
    mcap: StrictStr = Field(description="Market cap")
    price: StrictStr = Field(description="Price")
    liquidity: StrictStr = Field(description="The liquidity of the token in USD")
    risk: Risk = Field(description="Risk assessment")
    buys: StrictInt = Field(description="Number of buys")
    sells: StrictInt = Field(description="Number of sells")
    txns: StrictInt = Field(description="Total transactions")
    holders: StrictInt = Field(description="Number of holders")
    __properties: ClassVar[List[str]] = [
        "token",
        "mcap",
        "price",
        "liquidity",
        "risk",
        "buys",
        "sells",
        "txns",
        "holders",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of TokenData from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of token
        if self.token:
            _dict["token"] = self.token.to_dict()
        # override the default output from pydantic by calling `to_dict()` of risk
        if self.risk:
            _dict["risk"] = self.risk.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of TokenData from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "token": (
                    TokenDetail.from_dict(obj["token"])
                    if obj.get("token") is not None
                    else None
                ),
                "mcap": obj.get("mcap"),
                "price": obj.get("price"),
                "liquidity": obj.get("liquidity"),
                "risk": (
                    Risk.from_dict(obj["risk"]) if obj.get("risk") is not None else None
                ),
                "buys": obj.get("buys"),
                "sells": obj.get("sells"),
                "txns": obj.get("txns"),
                "holders": obj.get("holders"),
            }
        )
        return _obj



# flake8: noqa

# import apis into api package
from crypticorn.auth.client.api.admin_api import AdminApi
from crypticorn.auth.client.api.auth_api import AuthApi
from crypticorn.auth.client.api.user_api import UserApi
from crypticorn.auth.client.api.wallet_api import WalletApi



"""Defines common enumerations used throughout the codebase for type safety and consistency."""

try:
    from enum import StrEnum
except ImportError:
    from strenum import StrEnum

import warnings

from crypticorn.common.mixins import ValidateEnumMixin
from crypticorn.common.warnings import CrypticornDeprecatedSince215


class Exchange(ValidateEnumMixin, StrEnum):
    """All exchanges used in the crypticorn ecosystem. Refer to the APIs for support for a specific usecase (data, trading, etc.)."""

    KUCOIN = "kucoin"
    BINGX = "bingx"
    BINANCE = "binance"
    BYBIT = "bybit"
    HYPERLIQUID = "hyperliquid"
    BITGET = "bitget"
    GATEIO = "gateio"
    BITSTAMP = "bitstamp"


class InternalExchange(ValidateEnumMixin, StrEnum):
    """All exchanges we are using, including public (Exchange)"""

    KUCOIN = "kucoin"
    BINGX = "bingx"
    BINANCE = "binance"
    BYBIT = "bybit"
    HYPERLIQUID = "hyperliquid"
    BITGET = "bitget"

    @classmethod
    def __getattr__(cls, name):
        warnings.warn(
            "The `InternalExchange` enum is deprecated; use `Exchange` instead.",
            category=CrypticornDeprecatedSince215,
        )
        return super().__getattr__(name)


class MarketType(ValidateEnumMixin, StrEnum):
    """
    Market types
    """

    SPOT = "spot"
    FUTURES = "futures"



# coding: utf-8

"""
Trading API

API for automated trading and exchange interface. This API is used to trade on the exchange and manage bots, API keys, orders, and more.

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Awaitable, Dict, List, Optional, Tuple, Union

from pydantic import Field, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

from crypticorn.trade.client.api_client import ApiClient, RequestSerialized
from crypticorn.trade.client.api_response import ApiResponse
from crypticorn.trade.client.models.orders_count import OrdersCount
from crypticorn.trade.client.rest import RESTResponseType

# Import async_to_sync for sync methods
try:
    from asgiref.sync import async_to_sync

    _HAS_ASGIREF = True
except ImportError:
    _HAS_ASGIREF = False

    def async_to_sync(async_func):
        """Fallback decorator that raises an error if asgiref is not available."""

        def wrapper(*args, **kwargs):
            raise ImportError(
                "asgiref is required for sync methods. Install with: pip install asgiref"
            )

        return wrapper


class OrdersApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None, is_sync: bool = False) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client
        self.is_sync = is_sync

    @validate_call
    def get_orders_count(
        self,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        group_by: Annotated[
            Optional[StrictStr],
            Field(
                description="The group by period for the orders count. Defaults to day."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[List[OrdersCount], Awaitable[List[OrdersCount]]]:
        """Get Orders Count"""
        if self.is_sync:
            return self._get_orders_count_sync(
                sort_order=sort_order,
                sort_by=sort_by,
                group_by=group_by,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_orders_count_async(
                sort_order=sort_order,
                sort_by=sort_by,
                group_by=group_by,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_orders_count_with_http_info(
        self,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        group_by: Annotated[
            Optional[StrictStr],
            Field(
                description="The group by period for the orders count. Defaults to day."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[
        ApiResponse[List[OrdersCount]], Awaitable[ApiResponse[List[OrdersCount]]]
    ]:
        """Get Orders Count with HTTP info"""
        if self.is_sync:
            return self._get_orders_count_sync_with_http_info(
                sort_order=sort_order,
                sort_by=sort_by,
                group_by=group_by,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_orders_count_async_with_http_info(
                sort_order=sort_order,
                sort_by=sort_by,
                group_by=group_by,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_orders_count_without_preload_content(
        self,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        group_by: Annotated[
            Optional[StrictStr],
            Field(
                description="The group by period for the orders count. Defaults to day."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Orders Count without preloading content"""
        if self.is_sync:
            return self._get_orders_count_sync_without_preload_content(
                sort_order=sort_order,
                sort_by=sort_by,
                group_by=group_by,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_orders_count_async_without_preload_content(
                sort_order=sort_order,
                sort_by=sort_by,
                group_by=group_by,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_orders_count_async(
        self,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        group_by: Annotated[
            Optional[StrictStr],
            Field(
                description="The group by period for the orders count. Defaults to day."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[OrdersCount]:
        """Get Orders Count

        Get the number of orders for all users by day, week, month, or year. The default sort is `timestamp` and the default order is `asc`..

        :param sort_order: The order to sort by
        :type sort_order: str
        :param sort_by: The field to sort by
        :type sort_by: str
        :param group_by: The group by period for the orders count. Defaults to day.
        :type group_by: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_orders_count_serialize(
            sort_order=sort_order,
            sort_by=sort_by,
            group_by=group_by,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[OrdersCount]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_orders_count_async_with_http_info(
        self,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        group_by: Annotated[
            Optional[StrictStr],
            Field(
                description="The group by period for the orders count. Defaults to day."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[OrdersCount]]:
        """Get Orders Count

        Get the number of orders for all users by day, week, month, or year. The default sort is `timestamp` and the default order is `asc`..

        :param sort_order: The order to sort by
        :type sort_order: str
        :param sort_by: The field to sort by
        :type sort_by: str
        :param group_by: The group by period for the orders count. Defaults to day.
        :type group_by: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_orders_count_serialize(
            sort_order=sort_order,
            sort_by=sort_by,
            group_by=group_by,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[OrdersCount]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_orders_count_async_without_preload_content(
        self,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        group_by: Annotated[
            Optional[StrictStr],
            Field(
                description="The group by period for the orders count. Defaults to day."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Orders Count

        Get the number of orders for all users by day, week, month, or year. The default sort is `timestamp` and the default order is `asc`..

        :param sort_order: The order to sort by
        :type sort_order: str
        :param sort_by: The field to sort by
        :type sort_by: str
        :param group_by: The group by period for the orders count. Defaults to day.
        :type group_by: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_orders_count_serialize(
            sort_order=sort_order,
            sort_by=sort_by,
            group_by=group_by,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[OrdersCount]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_orders_count_sync(
        self,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        group_by: Annotated[
            Optional[StrictStr],
            Field(
                description="The group by period for the orders count. Defaults to day."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[OrdersCount]:
        """Synchronous version of get_orders_count"""
        return async_to_sync(self._get_orders_count_async)(
            sort_order=sort_order,
            sort_by=sort_by,
            group_by=group_by,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_orders_count_sync_with_http_info(
        self,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        group_by: Annotated[
            Optional[StrictStr],
            Field(
                description="The group by period for the orders count. Defaults to day."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[OrdersCount]]:
        """Synchronous version of get_orders_count_with_http_info"""
        return async_to_sync(self._get_orders_count_async_with_http_info)(
            sort_order=sort_order,
            sort_by=sort_by,
            group_by=group_by,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_orders_count_sync_without_preload_content(
        self,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        group_by: Annotated[
            Optional[StrictStr],
            Field(
                description="The group by period for the orders count. Defaults to day."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_orders_count_without_preload_content"""
        return async_to_sync(self._get_orders_count_async_without_preload_content)(
            sort_order=sort_order,
            sort_by=sort_by,
            group_by=group_by,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_orders_count_serialize(
        self,
        sort_order,
        sort_by,
        group_by,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if sort_order is not None:

            _query_params.append(("sort_order", sort_order))

        if sort_by is not None:

            _query_params.append(("sort_by", sort_by))

        if group_by is not None:

            _query_params.append(("group_by", group_by))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/orders/count",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



# coding: utf-8

"""
Payment API

API for accepting payments and storing subscriptions

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

from typing import Any, Awaitable, Dict, List, Optional, Tuple, Union

from pydantic import Field, StrictFloat, StrictInt, StrictStr, validate_call
from typing_extensions import Annotated

from crypticorn.pay.client.api_client import ApiClient, RequestSerialized
from crypticorn.pay.client.api_response import ApiResponse
from crypticorn.pay.client.models.paginated_response_payment import (
    PaginatedResponsePayment,
)
from crypticorn.pay.client.models.payment import Payment
from crypticorn.pay.client.models.subscription import Subscription
from crypticorn.pay.client.rest import RESTResponseType

# Import async_to_sync for sync methods
try:
    from asgiref.sync import async_to_sync

    _HAS_ASGIREF = True
except ImportError:
    _HAS_ASGIREF = False

    def async_to_sync(async_func):
        """Fallback decorator that raises an error if asgiref is not available."""

        def wrapper(*args, **kwargs):
            raise ImportError(
                "asgiref is required for sync methods. Install with: pip install asgiref"
            )

        return wrapper


class PaymentsApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None, is_sync: bool = False) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client
        self.is_sync = is_sync

    @validate_call
    def get_payment_history(
        self,
        filter_by: Annotated[
            Optional[StrictStr], Field(description="The field to filter by")
        ] = None,
        filter_value: Annotated[
            Optional[StrictStr], Field(description="The value to filter with")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="The current page number")
        ] = None,
        page_size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(
                description="The number of items per page. Default is 10, max is 100."
            ),
        ] = None,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[PaginatedResponsePayment, Awaitable[PaginatedResponsePayment]]:
        """Get Payment History"""
        if self.is_sync:
            return self._get_payment_history_sync(
                filter_by=filter_by,
                filter_value=filter_value,
                sort_order=sort_order,
                sort_by=sort_by,
                page=page,
                page_size=page_size,
                user_id=user_id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_payment_history_async(
                filter_by=filter_by,
                filter_value=filter_value,
                sort_order=sort_order,
                sort_by=sort_by,
                page=page,
                page_size=page_size,
                user_id=user_id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_payment_history_with_http_info(
        self,
        filter_by: Annotated[
            Optional[StrictStr], Field(description="The field to filter by")
        ] = None,
        filter_value: Annotated[
            Optional[StrictStr], Field(description="The value to filter with")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="The current page number")
        ] = None,
        page_size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(
                description="The number of items per page. Default is 10, max is 100."
            ),
        ] = None,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[
        ApiResponse[PaginatedResponsePayment],
        Awaitable[ApiResponse[PaginatedResponsePayment]],
    ]:
        """Get Payment History with HTTP info"""
        if self.is_sync:
            return self._get_payment_history_sync_with_http_info(
                filter_by=filter_by,
                filter_value=filter_value,
                sort_order=sort_order,
                sort_by=sort_by,
                page=page,
                page_size=page_size,
                user_id=user_id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_payment_history_async_with_http_info(
                filter_by=filter_by,
                filter_value=filter_value,
                sort_order=sort_order,
                sort_by=sort_by,
                page=page,
                page_size=page_size,
                user_id=user_id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_payment_history_without_preload_content(
        self,
        filter_by: Annotated[
            Optional[StrictStr], Field(description="The field to filter by")
        ] = None,
        filter_value: Annotated[
            Optional[StrictStr], Field(description="The value to filter with")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="The current page number")
        ] = None,
        page_size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(
                description="The number of items per page. Default is 10, max is 100."
            ),
        ] = None,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Payment History without preloading content"""
        if self.is_sync:
            return self._get_payment_history_sync_without_preload_content(
                filter_by=filter_by,
                filter_value=filter_value,
                sort_order=sort_order,
                sort_by=sort_by,
                page=page,
                page_size=page_size,
                user_id=user_id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_payment_history_async_without_preload_content(
                filter_by=filter_by,
                filter_value=filter_value,
                sort_order=sort_order,
                sort_by=sort_by,
                page=page,
                page_size=page_size,
                user_id=user_id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_payment_history_async(
        self,
        filter_by: Annotated[
            Optional[StrictStr], Field(description="The field to filter by")
        ] = None,
        filter_value: Annotated[
            Optional[StrictStr], Field(description="The value to filter with")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="The current page number")
        ] = None,
        page_size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(
                description="The number of items per page. Default is 10, max is 100."
            ),
        ] = None,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PaginatedResponsePayment:
        """Get Payment History

        Get the combined payment history for a user across all payment services.

        :param filter_by: The field to filter by
        :type filter_by: str
        :param filter_value: The value to filter with
        :type filter_value: str
        :param sort_order: The order to sort by
        :type sort_order: str
        :param sort_by: The field to sort by
        :type sort_by: str
        :param page: The current page number
        :type page: int
        :param page_size: The number of items per page. Default is 10, max is 100.
        :type page_size: int
        :param user_id: The ID of the user. Overrides the authenticated user if provided and the user is an admin.
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_payment_history_serialize(
            filter_by=filter_by,
            filter_value=filter_value,
            sort_order=sort_order,
            sort_by=sort_by,
            page=page,
            page_size=page_size,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PaginatedResponsePayment",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_payment_history_async_with_http_info(
        self,
        filter_by: Annotated[
            Optional[StrictStr], Field(description="The field to filter by")
        ] = None,
        filter_value: Annotated[
            Optional[StrictStr], Field(description="The value to filter with")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="The current page number")
        ] = None,
        page_size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(
                description="The number of items per page. Default is 10, max is 100."
            ),
        ] = None,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PaginatedResponsePayment]:
        """Get Payment History

        Get the combined payment history for a user across all payment services.

        :param filter_by: The field to filter by
        :type filter_by: str
        :param filter_value: The value to filter with
        :type filter_value: str
        :param sort_order: The order to sort by
        :type sort_order: str
        :param sort_by: The field to sort by
        :type sort_by: str
        :param page: The current page number
        :type page: int
        :param page_size: The number of items per page. Default is 10, max is 100.
        :type page_size: int
        :param user_id: The ID of the user. Overrides the authenticated user if provided and the user is an admin.
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_payment_history_serialize(
            filter_by=filter_by,
            filter_value=filter_value,
            sort_order=sort_order,
            sort_by=sort_by,
            page=page,
            page_size=page_size,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PaginatedResponsePayment",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_payment_history_async_without_preload_content(
        self,
        filter_by: Annotated[
            Optional[StrictStr], Field(description="The field to filter by")
        ] = None,
        filter_value: Annotated[
            Optional[StrictStr], Field(description="The value to filter with")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="The current page number")
        ] = None,
        page_size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(
                description="The number of items per page. Default is 10, max is 100."
            ),
        ] = None,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Payment History

        Get the combined payment history for a user across all payment services.

        :param filter_by: The field to filter by
        :type filter_by: str
        :param filter_value: The value to filter with
        :type filter_value: str
        :param sort_order: The order to sort by
        :type sort_order: str
        :param sort_by: The field to sort by
        :type sort_by: str
        :param page: The current page number
        :type page: int
        :param page_size: The number of items per page. Default is 10, max is 100.
        :type page_size: int
        :param user_id: The ID of the user. Overrides the authenticated user if provided and the user is an admin.
        :type user_id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_payment_history_serialize(
            filter_by=filter_by,
            filter_value=filter_value,
            sort_order=sort_order,
            sort_by=sort_by,
            page=page,
            page_size=page_size,
            user_id=user_id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "PaginatedResponsePayment",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_payment_history_sync(
        self,
        filter_by: Annotated[
            Optional[StrictStr], Field(description="The field to filter by")
        ] = None,
        filter_value: Annotated[
            Optional[StrictStr], Field(description="The value to filter with")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="The current page number")
        ] = None,
        page_size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(
                description="The number of items per page. Default is 10, max is 100."
            ),
        ] = None,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> PaginatedResponsePayment:
        """Synchronous version of get_payment_history"""
        return async_to_sync(self._get_payment_history_async)(
            filter_by=filter_by,
            filter_value=filter_value,
            sort_order=sort_order,
            sort_by=sort_by,
            page=page,
            page_size=page_size,
            user_id=user_id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_payment_history_sync_with_http_info(
        self,
        filter_by: Annotated[
            Optional[StrictStr], Field(description="The field to filter by")
        ] = None,
        filter_value: Annotated[
            Optional[StrictStr], Field(description="The value to filter with")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="The current page number")
        ] = None,
        page_size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(
                description="The number of items per page. Default is 10, max is 100."
            ),
        ] = None,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[PaginatedResponsePayment]:
        """Synchronous version of get_payment_history_with_http_info"""
        return async_to_sync(self._get_payment_history_async_with_http_info)(
            filter_by=filter_by,
            filter_value=filter_value,
            sort_order=sort_order,
            sort_by=sort_by,
            page=page,
            page_size=page_size,
            user_id=user_id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_payment_history_sync_without_preload_content(
        self,
        filter_by: Annotated[
            Optional[StrictStr], Field(description="The field to filter by")
        ] = None,
        filter_value: Annotated[
            Optional[StrictStr], Field(description="The value to filter with")
        ] = None,
        sort_order: Annotated[
            Optional[StrictStr], Field(description="The order to sort by")
        ] = None,
        sort_by: Annotated[
            Optional[StrictStr], Field(description="The field to sort by")
        ] = None,
        page: Annotated[
            Optional[StrictInt], Field(description="The current page number")
        ] = None,
        page_size: Annotated[
            Optional[Annotated[int, Field(le=100, strict=True, ge=1)]],
            Field(
                description="The number of items per page. Default is 10, max is 100."
            ),
        ] = None,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_payment_history_without_preload_content"""
        return async_to_sync(self._get_payment_history_async_without_preload_content)(
            filter_by=filter_by,
            filter_value=filter_value,
            sort_order=sort_order,
            sort_by=sort_by,
            page=page,
            page_size=page_size,
            user_id=user_id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_payment_history_serialize(
        self,
        filter_by,
        filter_value,
        sort_order,
        sort_by,
        page,
        page_size,
        user_id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if filter_by is not None:

            _query_params.append(("filter_by", filter_by))

        if filter_value is not None:

            _query_params.append(("filter_value", filter_value))

        if sort_order is not None:

            _query_params.append(("sort_order", sort_order))

        if sort_by is not None:

            _query_params.append(("sort_by", sort_by))

        if page is not None:

            _query_params.append(("page", page))

        if page_size is not None:

            _query_params.append(("page_size", page_size))

        if user_id is not None:

            _query_params.append(("user_id", user_id))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/payments/history",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_payments_by_invoice(
        self,
        id: Annotated[StrictStr, Field(description="The invoice ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[List[Payment], Awaitable[List[Payment]]]:
        """Get Payments By Invoice"""
        if self.is_sync:
            return self._get_payments_by_invoice_sync(
                id=id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_payments_by_invoice_async(
                id=id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_payments_by_invoice_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The invoice ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[ApiResponse[List[Payment]], Awaitable[ApiResponse[List[Payment]]]]:
        """Get Payments By Invoice with HTTP info"""
        if self.is_sync:
            return self._get_payments_by_invoice_sync_with_http_info(
                id=id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_payments_by_invoice_async_with_http_info(
                id=id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_payments_by_invoice_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The invoice ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Payments By Invoice without preloading content"""
        if self.is_sync:
            return self._get_payments_by_invoice_sync_without_preload_content(
                id=id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_payments_by_invoice_async_without_preload_content(
                id=id,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_payments_by_invoice_async(
        self,
        id: Annotated[StrictStr, Field(description="The invoice ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[Payment]:
        """Get Payments By Invoice

        Get all payments by invoice ID. Usually is just one payment, but can be multiple in case of partial payments.

        :param id: The invoice ID (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_payments_by_invoice_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Payment]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_payments_by_invoice_async_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The invoice ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[Payment]]:
        """Get Payments By Invoice

        Get all payments by invoice ID. Usually is just one payment, but can be multiple in case of partial payments.

        :param id: The invoice ID (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_payments_by_invoice_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Payment]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_payments_by_invoice_async_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The invoice ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Payments By Invoice

        Get all payments by invoice ID. Usually is just one payment, but can be multiple in case of partial payments.

        :param id: The invoice ID (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_payments_by_invoice_serialize(
            id=id,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Payment]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_payments_by_invoice_sync(
        self,
        id: Annotated[StrictStr, Field(description="The invoice ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[Payment]:
        """Synchronous version of get_payments_by_invoice"""
        return async_to_sync(self._get_payments_by_invoice_async)(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_payments_by_invoice_sync_with_http_info(
        self,
        id: Annotated[StrictStr, Field(description="The invoice ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[Payment]]:
        """Synchronous version of get_payments_by_invoice_with_http_info"""
        return async_to_sync(self._get_payments_by_invoice_async_with_http_info)(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_payments_by_invoice_sync_without_preload_content(
        self,
        id: Annotated[StrictStr, Field(description="The invoice ID")],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_payments_by_invoice_without_preload_content"""
        return async_to_sync(
            self._get_payments_by_invoice_async_without_preload_content
        )(
            id=id,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_payments_by_invoice_serialize(
        self,
        id,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params["id"] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = []

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/payments/by-invoice/{id}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def get_subscriptions(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        state: Annotated[
            Optional[StrictStr],
            Field(
                description="The state of the subscriptions to get. Defaults to all states."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[List[Subscription], Awaitable[List[Subscription]]]:
        """Get Subscriptions"""
        if self.is_sync:
            return self._get_subscriptions_sync(
                user_id=user_id,
                state=state,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_subscriptions_async(
                user_id=user_id,
                state=state,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_subscriptions_with_http_info(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        state: Annotated[
            Optional[StrictStr],
            Field(
                description="The state of the subscriptions to get. Defaults to all states."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[
        ApiResponse[List[Subscription]], Awaitable[ApiResponse[List[Subscription]]]
    ]:
        """Get Subscriptions with HTTP info"""
        if self.is_sync:
            return self._get_subscriptions_sync_with_http_info(
                user_id=user_id,
                state=state,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_subscriptions_async_with_http_info(
                user_id=user_id,
                state=state,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    @validate_call
    def get_subscriptions_without_preload_content(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        state: Annotated[
            Optional[StrictStr],
            Field(
                description="The state of the subscriptions to get. Defaults to all states."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> Union[RESTResponseType, Awaitable[RESTResponseType]]:
        """Get Subscriptions without preloading content"""
        if self.is_sync:
            return self._get_subscriptions_sync_without_preload_content(
                user_id=user_id,
                state=state,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

        else:
            return self._get_subscriptions_async_without_preload_content(
                user_id=user_id,
                state=state,
                _request_timeout=_request_timeout,
                _request_auth=_request_auth,
                _content_type=_content_type,
                _headers=_headers,
                _host_index=_host_index,
            )

    # Private async implementation methods
    @validate_call
    async def _get_subscriptions_async(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        state: Annotated[
            Optional[StrictStr],
            Field(
                description="The state of the subscriptions to get. Defaults to all states."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[Subscription]:
        """Get Subscriptions

        Get all subscriptions for a user. Subscriptions are the products a user has subscribed to. Returns both active and inactive subscriptions.

        :param user_id: The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin.
        :type user_id: str
        :param state: The state of the subscriptions to get. Defaults to all states.
        :type state: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_subscriptions_serialize(
            user_id=user_id,
            state=state,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Subscription]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def _get_subscriptions_async_with_http_info(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        state: Annotated[
            Optional[StrictStr],
            Field(
                description="The state of the subscriptions to get. Defaults to all states."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[Subscription]]:
        """Get Subscriptions

        Get all subscriptions for a user. Subscriptions are the products a user has subscribed to. Returns both active and inactive subscriptions.

        :param user_id: The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin.
        :type user_id: str
        :param state: The state of the subscriptions to get. Defaults to all states.
        :type state: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_subscriptions_serialize(
            user_id=user_id,
            state=state,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Subscription]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data, response_types_map=_response_types_map
        )

    @validate_call
    async def _get_subscriptions_async_without_preload_content(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        state: Annotated[
            Optional[StrictStr],
            Field(
                description="The state of the subscriptions to get. Defaults to all states."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Subscriptions

        Get all subscriptions for a user. Subscriptions are the products a user has subscribed to. Returns both active and inactive subscriptions.

        :param user_id: The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin.
        :type user_id: str
        :param state: The state of the subscriptions to get. Defaults to all states.
        :type state: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_subscriptions_serialize(
            user_id=user_id,
            state=state,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Subscription]",
        }

        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data

    # Private sync implementation methods
    @validate_call
    def _get_subscriptions_sync(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        state: Annotated[
            Optional[StrictStr],
            Field(
                description="The state of the subscriptions to get. Defaults to all states."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[Subscription]:
        """Synchronous version of get_subscriptions"""
        return async_to_sync(self._get_subscriptions_async)(
            user_id=user_id,
            state=state,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_subscriptions_sync_with_http_info(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        state: Annotated[
            Optional[StrictStr],
            Field(
                description="The state of the subscriptions to get. Defaults to all states."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[Subscription]]:
        """Synchronous version of get_subscriptions_with_http_info"""
        return async_to_sync(self._get_subscriptions_async_with_http_info)(
            user_id=user_id,
            state=state,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    @validate_call
    def _get_subscriptions_sync_without_preload_content(
        self,
        user_id: Annotated[
            Optional[StrictStr],
            Field(
                description="The ID of the user to get subscriptions for. Overrides the authenticated user if provided and the user is an admin."
            ),
        ] = None,
        state: Annotated[
            Optional[StrictStr],
            Field(
                description="The state of the subscriptions to get. Defaults to all states."
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Synchronous version of get_subscriptions_without_preload_content"""
        return async_to_sync(self._get_subscriptions_async_without_preload_content)(
            user_id=user_id,
            state=state,
            _request_timeout=_request_timeout,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

    def _get_subscriptions_serialize(
        self,
        user_id,
        state,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if user_id is not None:

            _query_params.append(("user_id", user_id))

        if state is not None:

            _query_params.append(("state", state))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/payments/subscriptions",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



from __future__ import annotations

from typing import TYPE_CHECKING, Optional

from crypticorn.auth import (
    AdminApi,
    ApiClient,
    AuthApi,
    Configuration,
    UserApi,
    WalletApi,
)

if TYPE_CHECKING:
    from aiohttp import ClientSession


class AuthClient:
    """
    A client for interacting with the Crypticorn Auth API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
        http_client: Optional[ClientSession] = None,
        is_sync: bool = False,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        if http_client is not None:
            self.base_client.rest_client.pool_manager = http_client
        # Pass sync context to REST client for proper session management
        self.base_client.rest_client.is_sync = is_sync
        # Instantiate all the endpoint clients
        self.admin = AdminApi(self.base_client, is_sync=is_sync)
        self.user = UserApi(self.base_client, is_sync=is_sync)
        self.wallet = WalletApi(self.base_client, is_sync=is_sync)
        self.login = AuthApi(self.base_client, is_sync=is_sync)
